@@ -146,6 +146,8 @@ typedef struct
   int upper; /*!< Upper bound. */
 } stp_int_bound_t;
 
+#define STP_CHANNEL_NONE ((unsigned char) -1)
+
 /** Parameter description. */
 typedef struct
 {
@@ -961,6 +963,88 @@ extern void stp_clear_array_parameter(stp_vars_t *v, const char *parameter);
  */
 extern void stp_clear_raw_parameter(stp_vars_t *v, const char *parameter);
 
+/**
+ * Clear a parameter.
+ * @param v the vars to use.
+ * @param parameter the name of the parameter.
+ * @param type the type of the parameter.
+ */
+extern void stp_clear_parameter(stp_vars_t *v, const char *parameter, stp_parameter_type_t type);
+
+
+/**
+ * List all string parameters.
+ * The return value must be freed after use.
+ * @param v the vars to use.
+ */
+extern stp_string_list_t *stp_list_string_parameters(const stp_vars_t *v);
+
+/**
+ * List all file parameters.
+ * The return value must be freed after use.
+ * @param v the vars to use.
+ */
+extern stp_string_list_t *stp_list_file_parameters(const stp_vars_t *v);
+
+/**
+ * List all float parameters.
+ * The return value must be freed after use.
+ * @param v the vars to use.
+ */
+extern stp_string_list_t *stp_list_float_parameters(const stp_vars_t *v);
+
+/**
+ * List all integer parameters.
+ * The return value must be freed after use.
+ * @param v the vars to use.
+ */
+extern stp_string_list_t *stp_list_int_parameters(const stp_vars_t *v);
+
+/**
+ * List all dimension parameters.
+ * The return value must be freed after use.
+ * @param v the vars to use.
+ */
+extern stp_string_list_t *stp_list_dimension_parameters(const stp_vars_t *v);
+
+/**
+ * List all boolean parameters.
+ * The return value must be freed after use.
+ * @param v the vars to use.
+ */
+extern stp_string_list_t *stp_list_boolean_parameters(const stp_vars_t *v);
+
+/**
+ * List all curve parameters.
+ * The return value must be freed after use.
+ * @param v the vars to use.
+ */
+extern stp_string_list_t *stp_list_curve_parameters(const stp_vars_t *v);
+
+/**
+ * List all array parameters.
+ * The return value must be freed after use.
+ * @param v the vars to use.
+ */
+extern stp_string_list_t *stp_list_array_parameters(const stp_vars_t *v);
+
+/**
+ * List all raw parameters.
+ * The return value must be freed after use.
+ * @param v the vars to use.
+ */
+extern stp_string_list_t *stp_list_raw_parameters(const stp_vars_t *v);
+
+/**
+ * List all parameters.
+ * The return value must be freed after use.
+ * @param v the vars to use.
+ * @param type the type of the parameter.
+ */
+extern stp_string_list_t *stp_list_parameters(const stp_vars_t *v,
+					      stp_parameter_type_t type);
+
+
 /**
  * Set the activity of a string parameter.
  * @param v the vars to use.
@@ -968,7 +1052,7 @@ extern void stp_clear_raw_parameter(stp_vars_t *v, const char *parameter);
  * @param active the activity status to set (should be set to
  * STP_PARAMETER_ACTIVE or STP_PARAMETER_INACTIVE).
  */
-extern void stp_set_string_parameter_active(const stp_vars_t *v,
+extern void stp_set_string_parameter_active(stp_vars_t *v,
 					    const char *parameter,
 					    stp_parameter_activity_t active);
 
@@ -979,7 +1063,7 @@ extern void stp_set_string_parameter_active(const stp_vars_t *v,
  * @param active the activity status to set (should be set to
  * STP_PARAMETER_ACTIVE or STP_PARAMETER_INACTIVE).
  */
-extern void stp_set_file_parameter_active(const stp_vars_t *v,
+extern void stp_set_file_parameter_active(stp_vars_t *v,
 					  const char *parameter,
 					  stp_parameter_activity_t active);
 
@@ -990,7 +1074,7 @@ extern void stp_set_file_parameter_active(const stp_vars_t *v,
  * @param active the activity status to set (should be set to
  * STP_PARAMETER_ACTIVE or STP_PARAMETER_INACTIVE).
  */
-extern void stp_set_float_parameter_active(const stp_vars_t *v,
+extern void stp_set_float_parameter_active(stp_vars_t *v,
 					 const char *parameter,
 					 stp_parameter_activity_t active);
 
@@ -1001,7 +1085,7 @@ extern void stp_set_float_parameter_active(const stp_vars_t *v,
  * @param active the activity status to set (should be set to
  * STP_PARAMETER_ACTIVE or STP_PARAMETER_INACTIVE).
  */
-extern void stp_set_int_parameter_active(const stp_vars_t *v,
+extern void stp_set_int_parameter_active(stp_vars_t *v,
 					 const char *parameter,
 					 stp_parameter_activity_t active);
 
@@ -1012,7 +1096,7 @@ extern void stp_set_int_parameter_active(const stp_vars_t *v,
  * @param active the activity status to set (should be set to
  * STP_PARAMETER_ACTIVE or STP_PARAMETER_INACTIVE).
  */
-extern void stp_set_dimension_parameter_active(const stp_vars_t *v,
+extern void stp_set_dimension_parameter_active(stp_vars_t *v,
 					       const char *parameter,
 					       stp_parameter_activity_t active);
 
@@ -1023,7 +1107,7 @@ extern void stp_set_dimension_parameter_active(const stp_vars_t *v,
  * @param active the activity status to set (should be set to
  * STP_PARAMETER_ACTIVE or STP_PARAMETER_INACTIVE).
  */
-extern void stp_set_boolean_parameter_active(const stp_vars_t *v,
+extern void stp_set_boolean_parameter_active(stp_vars_t *v,
 					     const char *parameter,
 					     stp_parameter_activity_t active);
 
@@ -1034,7 +1118,7 @@ extern void stp_set_boolean_parameter_active(const stp_vars_t *v,
  * @param active the activity status to set (should be set to
  * STP_PARAMETER_ACTIVE or STP_PARAMETER_INACTIVE).
  */
-extern void stp_set_curve_parameter_active(const stp_vars_t *v,
+extern void stp_set_curve_parameter_active(stp_vars_t *v,
 					   const char *parameter,
 					   stp_parameter_activity_t active);
 
@@ -1045,7 +1129,7 @@ extern void stp_set_curve_parameter_active(const stp_vars_t *v,
  * @param active the activity status to set (should be set to
  * STP_PARAMETER_ACTIVE or STP_PARAMETER_INACTIVE).
  */
-extern void stp_set_array_parameter_active(const stp_vars_t *v,
+extern void stp_set_array_parameter_active(stp_vars_t *v,
 					   const char *parameter,
 					   stp_parameter_activity_t active);
 
@@ -1056,10 +1140,23 @@ extern void stp_set_array_parameter_active(const stp_vars_t *v,
  * @param active the activity status to set (should be set to
  * STP_PARAMETER_ACTIVE or STP_PARAMETER_INACTIVE).
  */
-extern void stp_set_raw_parameter_active(const stp_vars_t *v,
+extern void stp_set_raw_parameter_active(stp_vars_t *v,
 					 const char *parameter,
 					 stp_parameter_activity_t active);
 
+/**
+ * Set the activity of a parameter.
+ * @param v the vars to use.
+ * @param parameter the name of the parameter.
+ * @param active the activity status to set (should be set to
+ * STP_PARAMETER_ACTIVE or STP_PARAMETER_INACTIVE).
+ * @param type the type of the parameter.
+ */
+extern void stp_set_parameter_active(stp_vars_t *v,
+				     const char *parameter,
+				     stp_parameter_activity_t active,
+				     stp_parameter_type_t type);
+
 /**
  * Check if a string parameter is set.
  * @param v the vars to use.
@@ -1141,6 +1238,17 @@ extern int stp_check_array_parameter(const stp_vars_t *v, const char *parameter,
 extern int stp_check_raw_parameter(const stp_vars_t *v, const char *parameter,
 				   stp_parameter_activity_t active);
 
+/**
+ * Check if a parameter is set.
+ * @param v the vars to use.
+ * @param parameter the name of the parameter.
+ * @param active the minimum activity status.
+ * @param type the type of the parameter.
+ */
+extern int stp_check_parameter(const stp_vars_t *v, const char *parameter,
+			       stp_parameter_activity_t active,
+			       stp_parameter_type_t type);
+
 /**
  * Get the activity status of a string parameter.
  * @param v the vars to use.
@@ -1222,6 +1330,16 @@ stp_get_array_parameter_active(const stp_vars_t *v, const char *parameter);
 extern stp_parameter_activity_t
 stp_get_raw_parameter_active(const stp_vars_t *v, const char *parameter);
 
+/**
+ * Get the activity status of a parameter.
+ * @param v the vars to use.
+ * @param parameter the name of the parameter.
+ * @param type the type of the parameter.
+ */
+extern stp_parameter_activity_t
+stp_get_parameter_active(const stp_vars_t *v, const char *parameter,
+			 stp_parameter_type_t type);
+
 
 
 /****************************************************************
@@ -1341,6 +1459,42 @@ extern int stp_verify(stp_vars_t *v);
  */
 extern const stp_vars_t *stp_default_settings(void);
 
+/**
+ * Get the value of a specified category for the specified parameter.
+ * @param v the vars to use.
+ * @param desc the parameter description to use (must already be described)
+ * @param category the name of the category to search for.
+ * @returns the value of the category or NULL.  String must be freed by caller.
+ */
+extern char *stp_parameter_get_category(const stp_vars_t *v,
+					const stp_parameter_t *desc,
+					const char *category);
+
+/**
+ * Determine whether a parameter has a category with the specified value.
+ * If a null value is passed in, return whether the parameter has
+ * the category at all.  Return -1 if any other error condition (null
+ * vars, desc, or category).
+ * @param v the vars to use.
+ * @param desc the parameter description to use (must already be described)
+ * @param category the name of the category to search for.
+ * @param value the value of the category to search for.
+ * @returns whether the parameter has the category with the specified value. 
+ */
+extern int stp_parameter_has_category_value(const stp_vars_t *v,
+					    const stp_parameter_t *desc,
+					    const char *category,
+					    const char *value);
+
+/**
+ * Get the list of categories and their values for the specified parameter.
+ * @param v the vars to use.
+ * @param desc the parameter description to use (must already be described)
+ * @returns the list of categories.
+ */
+extern stp_string_list_t *stp_parameter_get_categories(const stp_vars_t *v,
+						       const stp_parameter_t *desc);
+
 typedef void *(*stp_copy_data_func_t)(void *);
 typedef void (*stp_free_data_func_t)(void *);
 
@@ -1383,5 +1537,5 @@ stp_fill_parameter_settings(stp_parameter_t *desc,
 
 #endif /* GUTENPRINT_VARS_H */
 /*
- * End of "$Id: vars.h,v 1.4 2007/01/23 01:18:16 rlk Exp $".
+ * End of "$Id: vars.h,v 1.8 2010/12/05 21:38:14 rlk Exp $".
  */
