@@ -52,12 +52,16 @@
 
 
 #ifndef RDTIMEOUT
-#define RDTIMEOUT 10000
-#define WRTIMEOUT 10000
+#define RDTIMEOUT     10000
+#define WRTIMEOUT     10000
+#define RDDATATIMEOUT 1000
+#define MICROTIMEOUT  1
 #endif
 
-int d4WrTimeout = WRTIMEOUT;
-int d4RdTimeout = RDTIMEOUT;
+int d4WrTimeout     = WRTIMEOUT;
+int d4RdTimeout     = RDTIMEOUT;
+int d4RdDataTimeout = RDDATATIMEOUT;
+int d4MicroTimeout  = 1;
 int ppid        = 0;
 
 int debugD4     = 1;
@@ -67,6 +71,8 @@ static signalHandler_t sig;
 static int timeoutGot = 0;
 static int _readData(int fd, unsigned char *buf, int len);
 
+static int d4Errno = 0;
+
 /* commands for the D4 protocol
 
 Transaction    Cmd    Reply
@@ -195,7 +201,7 @@ static void printHexValues(const char *dir, const unsigned char *buf, int len)
 #if 0
    len = len > 30 ? 30 : len;
 #endif
-   fprintf(stderr,"%s\n",dir);
+   printf("%s\n",dir);
    for (i = 0; i < len; i++)
      {
        if (isprint(buf[i]))
@@ -219,11 +225,11 @@ static void printHexValues(const char *dir, const unsigned char *buf, int len)
      {
        for (i = 0; i < len; i++)
 	 {
-	   fprintf(stderr,"%c",isprint(buf[i])||isspace(buf[i])?buf[i]:'*');
+	   printf("%c",isprint(buf[i])||isspace(buf[i])?buf[i]:'*');
 	   if (buf[i] == ';' && i < len - 1)
-	     fprintf(stderr, "\n");
+	     printf("\n");
 	 }
-       fprintf(stderr, "\n");
+       printf("\n");
      }
    for (j = 0; j < blocks; j++)
      {
@@ -231,25 +237,25 @@ static void printHexValues(const char *dir, const unsigned char *buf, int len)
        int count = len;
        if (count > baseidx + 16)
 	 count =  baseidx + 16;
-       fprintf(stderr, "%4d: ", baseidx);
+       printf("%4d: ", baseidx);
        for ( i = baseidx; i < count;i++)
 	 {
 	   if (i % 4 == 0)
-	     fprintf(stderr, " ");
-	   fprintf(stderr," %02x",buf[i]);
+	     printf(" ");
+	   printf(" %02x",buf[i]);
 	 }
        if (print_strings)
 	 {
-	   fprintf(stderr,"\n      ");
+	   printf("\n      ");
 	   for ( i = baseidx; i < count;i++)
 	     {
 	       if (i % 4 == 0)
-		 fprintf(stderr, " ");
-	       fprintf(stderr,"  %c",
+		 printf(" ");
+	       printf("  %c",
 		       isprint(buf[i]) && !isspace(buf[i]) ? buf[i] : ' ');
 	     }
 	 }
-       fprintf(stderr, "\n");
+       printf("\n");
      }
 }
 
@@ -286,7 +292,7 @@ static void sigAlarm(int code)
 
 /*******************************************************************/
 /* Function printError()                                           */
-/*    print an error message on stderr                             */
+/*    print an error message on stdout                             */
 /*                                                                 */
 /* Input:  unsigned char errorNb the error number                  */
 /*                                                                 */
@@ -305,19 +311,20 @@ static int printError(unsigned char errorNb)
    {
       if ( msg->result == errorNb )
       {
-         fprintf(stderr,"%s\n", msg->message);
+	 if (debugD4)
+	   printf("%s\n", msg->message);
          return msg->errorClass;
       }
       msg++;
    }
-   fprintf(stderr,"Unknown IEEE 1284.4 error number %d\n",errorNb);
+   printf("Unknown IEEE 1284.4 error number %d\n",errorNb);
    return 0;
 }
 
 
 /*******************************************************************/
 /* Function printCmdType()                                         */
-/*        print on stderr the command name                         */
+/*        print on stdout the command name                         */
 /* Input:  unsigned char *cmd   the data are to be put here        */
 /*                                                                 */
 /* Return: -                                                       */
@@ -326,26 +333,28 @@ static int printError(unsigned char errorNb)
 
 static void printCmdType(unsigned char *cmd)
 {
+   if (cmd[6] & 0x80)
+     printf(">>>");
    if ( cmd[0] == 0 && cmd[1] == 0 )
    {
-      switch(cmd[6])
+      switch(cmd[6] & 0x7f)
       {
-         case    0: fprintf(stderr,"--- Init           ---\n");break;
-         case    1: fprintf(stderr,"--- OpenChannel    ---\n");break;
-         case    2: fprintf(stderr,"--- CloseChannel   ---\n");break;
-         case    3: fprintf(stderr,"--- Credit         ---\n");break;
-         case    4: fprintf(stderr,"--- CreditRequest  ---\n");break;
-         case    8: fprintf(stderr,"--- Exit           ---\n");break;
-         case    9: fprintf(stderr,"--- GetSocketID    ---\n");break;
-         case   10: fprintf(stderr,"--- GetServiceName ---\n");break;
-         case 0x45: fprintf(stderr,"--- EnterD4Mode    ---\n");break;
-         case 0x7f: fprintf(stderr,"--- Error          ---\n");break;
-         default:   fprintf(stderr,"--- ?????????????? ---\n");break;
+         case    0: printf("--- Init           ---\n");break;
+         case    1: printf("--- OpenChannel    ---\n");break;
+         case    2: printf("--- CloseChannel   ---\n");break;
+         case    3: printf("--- Credit         ---\n");break;
+         case    4: printf("--- CreditRequest  ---\n");break;
+         case    8: printf("--- Exit           ---\n");break;
+         case    9: printf("--- GetSocketID    ---\n");break;
+         case   10: printf("--- GetServiceName ---\n");break;
+         case 0x45: printf("--- EnterD4Mode    ---\n");break;
+         case 0x7f: printf("--- Error          ---\n");break;
+         default:   printf("--- ?????????????? ---\n");break;
       }
    }
    else
    {
-      fprintf(stderr,"--- Send Data      ---\n");
+      printf("--- Send Data      ---\n");
    }
 }
 
@@ -387,8 +396,9 @@ static int writeCmd(int fd, unsigned char *cmd, int len)
       }
    }
 
-   usleep(1); /* according to Glen Steward, this will solve problems  */
-              /* for the cartridge exchange with the Stylus Color 580 */
+   /* according to Glen Steward, this will solve problems  */
+   /* for the cartridge exchange with the Stylus Color 580 */
+   usleep(d4MicroTimeout);
 
    timeoutGot = 0;
    errno = 0;
@@ -416,7 +426,7 @@ static int writeCmd(int fd, unsigned char *cmd, int len)
       gettimeofday(&end, NULL);
       dt = (end.tv_sec  - beg.tv_sec) * 1000000;
       dt += end.tv_usec - beg.tv_usec;
-      fprintf(stderr,"Write time %5.3f s\n",(double)dt/1000000);
+      printf("Write time %5.3f s\n",(double)dt/1000000);
 # endif
    }
 
@@ -436,7 +446,7 @@ static int writeCmd(int fd, unsigned char *cmd, int len)
 /*                                                                 */
 /*******************************************************************/
 
-int readAnswer(int fd, unsigned char *buf, int len)
+int readAnswer(int fd, unsigned char *buf, int len, int allowExtra)
 {
    int rd    = 0;
    int total = 0;
@@ -445,6 +455,7 @@ int readAnswer(int fd, unsigned char *buf, int len)
    long dt;
    int count = 0;
    int first_read = 1;
+   int excess = 0;
    /* wait a little bit before reading an answer */
    usleep(d4RdTimeout);
 
@@ -458,7 +469,7 @@ int readAnswer(int fd, unsigned char *buf, int len)
    gettimeofday(&beg, NULL);
 
    if (debugD4)
-     fprintf(stderr, "length: %i\n", len);
+     printf("length: %i\n", len);
    while ( total < len )
    {
       SET_TIMER(ti,oti, d4RdTimeout);
@@ -467,16 +478,16 @@ int readAnswer(int fd, unsigned char *buf, int len)
 	{
 	  if (first_read)
 	    {
-	      fprintf(stderr, "read: ");
+	      printf("read: ");
 	      first_read = 0;
 	    }
 	  if (rd < 0)
 	    {
-	      fprintf(stderr, "%i %s\n", rd, errno != 0 ?strerror(errno) : "");
+	      printf("%i %s\n", rd, errno != 0 ?strerror(errno) : "");
 	      first_read = 1;
 	    }
 	  else
-	    fprintf(stderr, "%i ", rd);
+	    printf("%i ", rd);
 	}
       RESET_TIMER(ti,oti);
       if ( rd <= 0 )
@@ -487,7 +498,7 @@ int readAnswer(int fd, unsigned char *buf, int len)
          if ( dt > d4RdTimeout * 2 )
          {
             if ( debugD4 )
-               fprintf(stderr,"Timeout 1 at readAnswer() rcv %d bytes\n",total);
+               printf("Timeout 1 at readAnswer() rcv %d bytes\n",total);
             timeoutGot = 1;
             break;
          }
@@ -508,30 +519,66 @@ int readAnswer(int fd, unsigned char *buf, int len)
             /* in case of errors this may differ from   */
             /* the expected lenght. Setting len to this */
             /* value will avoid waiting for timeout     */
-	    len = (buf[2] << 8) + buf[3];
-	    len = (len > sizeof(buf))?sizeof(buf) - 1:len;
+	    int newlen = (buf[2] << 8) + buf[3];
+	    if (len > newlen)
+	      {
+		if (debugD4)
+		  printf("Changing len from %d to %d\n", len, newlen);
+		len = newlen;
+	      }
+	    else if (len < newlen)
+	      {
+		excess = newlen - len;
+		if (debugD4)
+		  printf("Expected %d, getting %d, %sflushing %d\n",
+			  len, newlen, allowExtra ? "not " : "", excess);
+	      }
          }
       }
       usleep(d4RdTimeout);
    }
+   if (! allowExtra)
+     {
+       int retry_count = 0;
+       while (excess > 0)
+	 {
+	   char wastebuf[256];
+	   int bytes = excess > 256 ? 256 : excess;
+	   int status = read(fd, wastebuf, bytes);
+	   if (status < 0)
+	     break;
+	   else if (status == 0 && retry_count > 2)
+	     break;
+	   else if (status == 0)
+	     retry_count++;
+	   else
+	     retry_count = 0;
+	   if (status < bytes)
+	     usleep(d4RdTimeout);
+	   if (debugD4)
+	     printHexValues("waste", (const unsigned char *) wastebuf, status);
+	   excess -= status;
+	 }
+     }
    
    if ( debugD4 )
    {
+      printCmdType(buf);
 #  if PTIME
       gettimeofday(&end, NULL);
 #  endif
-      fprintf(stderr, "total: %i\n", total);
+      printf("total: %i\n", total);
       printHexValues("Recv: ",buf,total);
 #  if PTIME
       dt = (end.tv_sec  - beg.tv_sec) * 1000000;
       dt += end.tv_usec - beg.tv_usec;
-      fprintf(stderr,"Read time %5.3f s\n",(double)dt/1000000);
+      printf("Read time %5.3f s\n",(double)dt/1000000);
 #  endif
    }
    if ( timeoutGot )
    {
       if ( debugD4 )
-         fprintf(stderr,"Timeout 2 at readAnswer()\n");
+         printf("Timeout 2 at readAnswer()\n");
       return -1;
    }
    return total;
@@ -554,14 +601,14 @@ static void _flushData(int fd)
    errno = 0;
 
    if (debugD4)
-     fprintf(stderr, "flush data: length: %i\n", len);
+     printf("flush data: length: %i\n", len);
    do
      {
        usleep(d4RdTimeout);
        SET_TIMER(ti,oti, d4RdTimeout);
        rd = read(fd, buf, len);
        if (debugD4)
-	 fprintf(stderr, "flush: read: %i %s\n", rd,
+	 printf("flush: read: %i %s\n", rd,
 		 rd < 0 && errno != 0 ?strerror(errno) : "");
        RESET_TIMER(ti,oti);
        count--;
@@ -608,7 +655,7 @@ static int _readData(int fd, unsigned char *buf, int len)
          if ( dt > d4RdTimeout*3 )
          {
             if ( debugD4 )
-               fprintf(stderr,"Timeout at _readData(), dt = %ld ms\n", dt);
+               printf("Timeout at _readData(), dt = %ld ms\n", dt);
             return -1;
             break;
          }
@@ -627,7 +674,7 @@ static int _readData(int fd, unsigned char *buf, int len)
    {
       toGet = (header[2] >> 8) + header[3] - 6;
       if (debugD4)
-	fprintf(stderr, "toGet: %i\n", toGet);
+	printf("toGet: %i\n", toGet);
       total = 0;
       gettimeofday(&beg, NULL);
       while ( total < toGet )
@@ -643,7 +690,7 @@ static int _readData(int fd, unsigned char *buf, int len)
             if ( dt > d4RdTimeout*3 )
             {
                if ( debugD4 )
-                  fprintf(stderr,"Timeout at _readData(), dt = %ld ms\n",dt);
+                  printf("Timeout at _readData(), dt = %ld ms\n",dt);
                return -1;
                break;
             }
@@ -673,15 +720,16 @@ static int _readData(int fd, unsigned char *buf, int len)
 /*                                                                 */
 /*******************************************************************/
 
-static int sendReceiveCmd(int fd, unsigned char *cmd, int len, unsigned char *answer, int expectedlen)
+static int sendReceiveCmd(int fd, unsigned char *cmd, int len, unsigned char *answer, int expectedlen, int allowExtra)
 {
    int rd;
+   d4Errno = 0;
    if ( (rd = writeCmd(fd, cmd, len ) ) != len )
    {
       if ( rd < 0 ) return -1;
       return 0;
    }
-   rd = readAnswer(fd, answer, expectedlen );
+   rd = readAnswer(fd, answer, expectedlen, allowExtra );
    if ( rd == 0 )
    {
       /* no answer from device */
@@ -691,7 +739,7 @@ static int sendReceiveCmd(int fd, unsigned char *cmd, int len, unsigned char *an
    {
       /* interrupted write call */
       if ( debugD4 )
-         fprintf(stderr,"interrupt received\n");
+         printf("interrupt received\n");
       return -1;
    }
    else
@@ -700,10 +748,12 @@ static int sendReceiveCmd(int fd, unsigned char *cmd, int len, unsigned char *an
       if ( answer[6] == 0x7f )
       {
          printError(answer[9]);
+	 d4Errno = answer[9];
          return -1;
       }
       else if (  answer[7] != 0 )
       {
+	 d4Errno = answer[7];
          if ( printError(answer[7]) )
          {
             return -1;
@@ -742,14 +792,18 @@ Loop:
    {
       return 0;
    }
-   rd = readAnswer(fd, buf, 8);
+   rd = readAnswer(fd, buf, 8, 0);
    if ( rd == 0 )
    {
       /* no answer from device */
+      if (debugD4)
+        printf(">>>No answer from printer\n");
       return 0;
    }
    else if ( rd < 0 )
    {
+      if (debugD4)
+        printf(">>>Interrupted write\n");
       /* interrupted write call */
       return 0;
    }
@@ -789,7 +843,7 @@ int Init(int fd)
    cmd.head.command  = 0;
    cmd.revision      = 0x10;
     
-   rd = sendReceiveCmd(fd, (unsigned char*)&cmd, sizeof(cmd), buf, 9 );
+   rd = sendReceiveCmd(fd, (unsigned char*)&cmd, sizeof(cmd), buf, 9, 0 );
    return rd == 9 ? 1 : 0;
 }
 
@@ -815,7 +869,7 @@ int Exit(int fd)
    cmd.control  = 0;
    cmd.command  = 8;
 
-   rd = sendReceiveCmd(fd, (unsigned char*)&cmd, sizeof(cmd), buf, 8 );
+   rd = sendReceiveCmd(fd, (unsigned char*)&cmd, sizeof(cmd), buf, 8, 0 );
    return rd > 0 ? 1 : rd;
 }
 
@@ -846,7 +900,7 @@ int GetSocketID(int fd, const char *serviceName)
    cmd->command  = 0x09;
    strcpy(buf + sizeof(cmdHeader_t), serviceName);
 
-   rd = sendReceiveCmd(fd, (unsigned char*)buf, len, rBuf, len + 2);
+   rd = sendReceiveCmd(fd, (unsigned char*)buf, len, rBuf, len + 2, 0);
    if ( rd > 0 )
    {
       return rBuf[8];
@@ -873,8 +927,9 @@ int OpenChannel(int fd, unsigned char sockId, int *sndSz, int *rcvSz)
    unsigned char  cmd[17];
    unsigned char  buf[20];
    int rd;
+   int i;
 
-   for(;;)
+   for(i = 0; i < 5; i++)	/* Retry count */
    {
       cmd[0]  = 0;       /* transaction sockets */
       cmd[1]  = 0;
@@ -894,10 +949,20 @@ int OpenChannel(int fd, unsigned char sockId, int *sndSz, int *rcvSz)
       cmd[15] = 0;    /* initial credit for us ? */
       cmd[16] = 0;
 
-      rd = sendReceiveCmd(fd, cmd, 17, buf, 16);
+      rd = sendReceiveCmd(fd, cmd, 17, buf, 16, 0);
       if ( rd == -1 )
       {
-         return -1;
+	if (debugD4)
+	  printf("OpenChannel %d fails, error %d\n", sockId, d4Errno);
+	if (d4Errno == 6)	/* channel already open */
+	  {
+	    if ( debugD4 )
+	      printf("Channel %d already open, closing\n", sockId);
+	    CloseChannel(fd, sockId);
+	    continue;
+	  }
+	else
+	  return -1;
       }
       else if ( rd == 16 )
       {
@@ -908,6 +973,8 @@ int OpenChannel(int fd, unsigned char sockId, int *sndSz, int *rcvSz)
          }
          else if ( buf[7] != 0 )
          {
+	   if (debugD4)
+	     printf("OpenChannel %d fails, hard error\n", sockId);
             /* hard error */
             return -1;
          }
@@ -917,7 +984,16 @@ int OpenChannel(int fd, unsigned char sockId, int *sndSz, int *rcvSz)
       }
       else
       {
+	if (d4Errno == 6)	/* channel already open */
+	  {
+	    if ( debugD4 )
+	      printf("Channel %d already open, closing\n", sockId);
+	    CloseChannel(fd, sockId);
+	    continue;
+	  }
          /* at this stage we can only have an error */
+	if (debugD4)
+	  printf("OpenChannel %d fails, wrong count %d\n", sockId, rd);
          return -1;
       }
    }
@@ -949,7 +1025,7 @@ int CloseChannel(int fd, unsigned char socketID)
    buf[sizeof(cmdHeader_t)+0] = socketID;
    buf[sizeof(cmdHeader_t)+1] = socketID;
    buf[sizeof(cmdHeader_t)+2] = 0;
-   rd = sendReceiveCmd(fd, buf,10, buf, 10);
+   rd = sendReceiveCmd(fd, buf,10, buf, 10, 0);
    return rd == 10 ? 1 : rd;
 }
 
@@ -982,7 +1058,7 @@ int CreditRequest(int fd, unsigned char socketID)
    buf[sizeof(cmdHeader_t)+3] = 0x80;
    buf[sizeof(cmdHeader_t)+4] = 0xff;
    buf[sizeof(cmdHeader_t)+5] = 0xff;
-   rd = sendReceiveCmd(fd, buf, 13, rBuf, 12);
+   rd = sendReceiveCmd(fd, buf, 13, rBuf, 12, 0);
    if ( rd == 12 )
    {
       /* this is the credit */
@@ -1023,7 +1099,7 @@ int Credit(int fd, unsigned char socketID, int credit)
    buf[sizeof(cmdHeader_t)+1] = socketID;
    buf[sizeof(cmdHeader_t)+2] = credit >> 8;
    buf[sizeof(cmdHeader_t)+3] = credit & 0xff;
-   rd = sendReceiveCmd(fd, buf, 11, rBuf, 10);
+   rd = sendReceiveCmd(fd, buf, 11, rBuf, 10, 0);
    if ( rd == 10 )
    {
       return 1;
@@ -1039,7 +1115,7 @@ int Credit(int fd, unsigned char socketID, int credit)
 /*        Convenience function                                     */
 /*        handle the CreditRequest command                         */
 /* Input:  int   fd    file handle                                 */
-/*         unsigned char    socketID                                          */
+/*         unsigned char    socketID                               */
 /* IN/Out  int  *sndSize  for error handling                       */
 /* IN/Out  int  *rcvSize  for error handling                       */
 /*                                                                 */
@@ -1054,10 +1130,11 @@ int askForCredit(int fd, unsigned char socketID, int *sndSize, int *rcvSize)
 {
    int credit = 0;
    int count  = 0;
+   int retries = 10;
    
-   while (credit == 0 )
+   while (credit == 0 && retries-- >= 0 )
    {
-      while((credit=CreditRequest(fd,socketID)) == 0  && count < MAX_CREDIT_REQUEST )
+      while((credit=CreditRequest(fd,socketID)) == 0  && count < MAX_CREDIT_REQUEST && retries-- >= 0)
          usleep(d4RdTimeout);
 
       if ( credit == -1 )
@@ -1069,8 +1146,11 @@ int askForCredit(int fd, unsigned char socketID, int *sndSize, int *rcvSize)
          credit = 0;
          /* init printer and reopen the printer channel */
          CloseChannel(fd, socketID);
+	 socketID = GetSocketID(fd, "EPSON-CTRL");
          if ( Init(fd) )
          {
+	   if (debugD4)
+	     printf("askForCredit init succeeded, now try to open\n");
             OpenChannel(fd, socketID, sndSize, rcvSize);
          }
       }
@@ -1087,8 +1167,8 @@ int askForCredit(int fd, unsigned char socketID, int *sndSize, int *rcvSize)
 /*        Convenience function                                     */
 /*        write the data to the device                             */
 /* Input:  int   fd    file handle                                 */
-/*         unsigned char    socketID  the deetination socket                  */
-/*         unsigned char   *buf       the datas to be send                    */
+/*         unsigned char    socketID  the deetination socket       */
+/*         unsigned char   *buf       the datas to be send         */
 /*         int   len       how many datas are to we send           */
 /*         int   eoj       set out of band flag if eoj set         */
 /*                                                                 */
@@ -1107,7 +1187,7 @@ int writeData(int fd, unsigned char socketID, const unsigned char *buf, int len,
    static int bLen   = 0;
    if ( debugD4 )
    {
-      fprintf(stderr,"--- Send Data      ---\n");
+      printf("--- Send Data      ---\n");
       gettimeofday(&beg, NULL);
    }
    len += 6;
@@ -1152,15 +1232,15 @@ int writeData(int fd, unsigned char socketID, const unsigned char *buf, int len,
       dt = (end.tv_sec  - beg.tv_sec) * 1000000;
       dt += end.tv_usec - beg.tv_usec;
 # endif  
-      fprintf(stderr,"Send: ");
+      printf("Send: ");
       for ( ret = 0; (wr > 0) && (ret < ((wr > 20) ? 20 : wr)) ; ret++ )
-         fprintf(stderr,"%02x ", buffer[ret]);
-      fprintf(stderr,"\n      ");
+         printf("%02x ", buffer[ret]);
+      printf("\n      ");
       for ( ret = 0; (wr > 0) && (ret < ((wr > 20) ? 20 : wr)) ; ret++ )
-         fprintf(stderr,"%c  ", isprint(buffer[ret])&&!isspace(buffer[ret])?buffer[ret]:' ');
-      fprintf(stderr,"\n");
+         printf("%c  ", isprint(buffer[ret])&&!isspace(buffer[ret])?buffer[ret]:' ');
+      printf("\n");
 # if PTIME
-       fprintf(stderr,"Write time %5.3f s\n",(double)dt/1000000);
+       printf("Write time %5.3f s\n",(double)dt/1000000);
 # endif
    }
 
@@ -1177,11 +1257,11 @@ int writeData(int fd, unsigned char socketID, const unsigned char *buf, int len,
 /*        Convenience function                                     */
 /*        give credit and read then the expected datas             */
 /* Input:  int   fd    file handle                                 */
-/*         unsigned char    socketID  the destination socket                  */
-/*         unsigned char   *buf       the datas to be send                    */
+/*         unsigned char    socketID  the destination socket       */
+/*         unsigned char   *buf       the datas to be send         */
 /*         int   len       howmany datas are to we send            */
 /*                                                                 */
-/* Return: number of bytes written or -1;                          */
+/* Return: number of bytes read or -1;                             */
 /*                                                                 */
 /*******************************************************************/
 
@@ -1192,7 +1272,7 @@ int readData(int fd, unsigned char socketID, unsigned char *buf, int len)
    if ( Credit(fd, socketID, 1) == 1 )
    {
       /* wait a little bit */
-      usleep(1000);
+      usleep(d4RdDataTimeout);
       ret = _readData(fd, buf, len);
       return ret; 
    }
@@ -1202,6 +1282,58 @@ int readData(int fd, unsigned char socketID, unsigned char *buf, int len)
    }
 }
 
+/*******************************************************************/
+/* Function writeAndReadData()                                     */
+/*        Convenience function                                     */
+/*        give credit and read then the expected datas             */
+/* Input:  int   fd    file handle                                 */
+/*         unsigned char    socketID  the destination socket       */
+/*         unsigned char   *cmd       the datas to be send         */
+/*         int   cmd_len       howmany datas are to we send        */
+/*         int   eoj       set out of band flag if eoj set         */
+/*         unsigned char   *buf       the datas to be send         */
+/*         int   *sndSz    Send buffer size                        */
+/*         int   *rcvSz    Receive buffer size                     */
+/*         int   len       how many datas are to we send           */
+/*         fptr  test      function to verify buffer contents      */ 
+/*                                                                 */
+/* Return: number of bytes read or -1;                             */
+/*                                                                 */
+/* This allows us to give credit before sending the command.       */
+/* Sending the command and then giving credit sometimes causes     */
+/* the actual data to be sent as a reply to the credit command.    */
+/*                                                                 */
+/*******************************************************************/
+
+int writeAndReadData(int fd, unsigned char socketID,
+		     const unsigned char *cmd, int cmd_len, int eoj,
+		     unsigned char *buf, int len, int *sndSz, int *rcvSz,
+		     int (*test)(const unsigned char *buf))
+{
+   int ret;
+   int retry = 5;
+   int credit = askForCredit(fd, socketID, sndSz, rcvSz);
+   if (credit < 0)
+     return -1;
+   /* give credit */
+   if ( Credit(fd, socketID, 1) == 1 )
+     {
+       if (writeData(fd, socketID, cmd, cmd_len, eoj) <= 0)
+	 return -1;
+       /* wait a little bit */
+       do
+	 {
+	   usleep(d4RdDataTimeout);
+	   ret = _readData(fd, buf, len);
+	   if (ret < 0)
+	     return ret;
+	 } while (retry-- >= 0 && (!test || !(*test)(buf)));
+       return ret;
+     }
+   else
+     return -1;
+}
+
 /*******************************************************************/
 /* Function readData()                                             */
 /*        Convenience function                                     */
@@ -1218,14 +1350,14 @@ int readData(int fd, unsigned char socketID, unsigned char *buf, int len)
 void flushData(int fd, unsigned char socketID)
 {
   if (debugD4)
-    fprintf(stderr, "flushData %d\n", socketID);
+    printf("flushData %d\n", socketID);
    /* give credit */
    if (socketID != (unsigned char) -1)
      {
        if ( Credit(fd, socketID, 1) == 1 )
 	 {
 	   /* wait a little bit */
-	   usleep(1000);
+	   usleep(d4RdDataTimeout);
 	   _flushData(fd);
 	 }
      }
