@@ -1,5 +1,5 @@
 /*
- * "$Id: escputil.c,v 1.97 2008/10/29 00:10:42 easysw Exp $"
+ * "$Id: escputil.c,v 1.104 2010/12/11 22:04:07 rlk Exp $"
  *
  *   Printer maintenance utility for EPSON Stylus (R) printers
  *
@@ -264,7 +264,7 @@ exit_packet_mode_old(int do_init)
 {
   static char hdr[] = "\000\000\000\033\001@EJL 1284.4\n@EJL     \n\033@";
   memcpy(printer_cmd + bufpos, hdr, sizeof(hdr) - 1); /* DON'T include null! */
-  STP_DEBUG(fprintf(stderr, "Exit packet mode (%d)\n", do_init));
+  STP_DEBUG(printf("Exit packet mode (%d)\n", do_init));
   bufpos += sizeof(hdr) - 1;
   if (!do_init)
     bufpos -= 2;
@@ -274,11 +274,19 @@ static void
 initialize_print_cmd(int do_init)
 {
   bufpos = 0;
-  STP_DEBUG(fprintf(stderr, "Initialize print command\n"));
+  STP_DEBUG(printf("Initialize print command\n"));
   if (isnew)
     exit_packet_mode_old(do_init);
 }
 
+static void
+initialize_print_cmd_new(int do_init)
+{
+  bufpos = 0;
+  STP_DEBUG(printf("Initialize print command (force new)\n"));
+  exit_packet_mode_old(do_init);
+}
+
 int
 main(int argc, char **argv)
 {
@@ -373,6 +381,7 @@ main(int argc, char **argv)
 	      printf(_("Alignment choices must be at least 1."));
 	      do_help(1);
 	    }
+	  break;
 	case 'p':
 	  alignment_passes = atoi(optarg);
 	  if (alignment_passes < 1)
@@ -380,6 +389,7 @@ main(int argc, char **argv)
 	      printf(_("Alignment passes must be at least 1."));
 	      do_help(1);
 	    }
+	  break;
 	default:
 	  printf("%s\n", gettext(banner));
 	  fprintf(stderr, _("Unknown option %c\n"), c);
@@ -435,15 +445,15 @@ print_debug_data(const char *buf, size_t count)
   for (i = 0; i < count; i++)
     {
       if (i % 16 == 0)
-	fprintf(stderr, "\n%4d: ", i);
+	printf("\n%4d: ", i);
       else if (i % 4 == 0)
-	fprintf(stderr, " ");
+	printf(" ");
       if (isgraph(buf[i]))
-	fprintf(stderr, "  %c", (unsigned) ((unsigned char) buf[i]));
+	printf("  %c", (unsigned) ((unsigned char) buf[i]));
       else
-	fprintf(stderr, " %02x", (unsigned) ((unsigned char) buf[i]));
+	printf(" %02x", (unsigned) ((unsigned char) buf[i]));
     }
-  fprintf(stderr, "\n");
+  printf("\n");
 }
 
 int
@@ -488,7 +498,7 @@ do_print_cmd(void)
 	  return 1;
 	}
     }
-  STP_DEBUG(fprintf(stderr, "Sending print command to %s:",
+  STP_DEBUG(printf("Sending print command to %s:",
 		    raw_device ? raw_device : command));
   STP_DEBUG(print_debug_data(printer_cmd, bufpos));
   while (bytes < bufpos)
@@ -539,7 +549,7 @@ read_from_printer(int fd, char *buf, int bufsize, int quiet)
   int retry = 100;
 #ifdef HAVE_FCNTL_H
   fcntl(fd, F_SETFL,
-	O_NONBLOCK | fcntl(fd, F_GETFL));
+	O_NONBLOCK | (fcntl(fd, F_GETFL) & ~(O_RDONLY|O_WRONLY|O_RDWR)));
 #endif
   memset(buf, 0, bufsize);
 
@@ -563,7 +573,7 @@ read_from_printer(int fd, char *buf, int bufsize, int quiet)
 
   if (status > 0)
     {
-      STP_DEBUG(fprintf(stderr, "read_from_printer returns %d\n", status));
+      STP_DEBUG(printf("read_from_printer returns %d\n", status));
       STP_DEBUG(print_debug_data(buf, status));
     }
   else if (status == 0 && retry == 0)
@@ -586,7 +596,7 @@ start_remote_sequence(void)
   static char remote_hdr[] = "\033@\033(R\010\000\000REMOTE1";
   memcpy(printer_cmd + bufpos, remote_hdr, sizeof(remote_hdr) - 1);
   bufpos += sizeof(remote_hdr) - 1;
-  STP_DEBUG(fprintf(stderr, "Start remote sequence\n"));
+  STP_DEBUG(printf("Start remote sequence\n"));
 }
 
 static void
@@ -595,7 +605,7 @@ end_remote_sequence(void)
   static char remote_trailer[] = "\033\000\000\000\033\000";
   memcpy(printer_cmd + bufpos, remote_trailer, sizeof(remote_trailer) - 1);
   bufpos += sizeof(remote_trailer) - 1;
-  STP_DEBUG(fprintf(stderr, "End remote sequence\n"));
+  STP_DEBUG(printf("End remote sequence\n"));
 }
 
 static void
@@ -607,21 +617,21 @@ do_remote_cmd(const char *cmd, int nargs, ...)
 
   start_remote_sequence();
   memcpy(printer_cmd + bufpos, cmd, 2);
-  STP_DEBUG(fprintf(stderr, "Remote command: %s", cmd));
+  STP_DEBUG(printf("Remote command: %s", cmd));
   bufpos += 2;
   printer_cmd[bufpos] = nargs % 256;
   printer_cmd[bufpos + 1] = (nargs >> 8) % 256;
-  STP_DEBUG(fprintf(stderr, " %02x %02x",
+  STP_DEBUG(printf(" %02x %02x",
 		    (unsigned) printer_cmd[bufpos],
 		    (unsigned) printer_cmd[bufpos + 1]));
   if (nargs > 0)
     for (i = 0; i < nargs; i++)
       {
 	printer_cmd[bufpos + 2 + i] = va_arg(args, int);
-	STP_DEBUG(fprintf(stderr, " %02x",
+	STP_DEBUG(printf(" %02x",
 			  (unsigned) printer_cmd[bufpos + 2 + i]));
       }
-  STP_DEBUG(fprintf(stderr, "\n"));
+  STP_DEBUG(printf("\n"));
   bufpos += 2 + nargs;
   end_remote_sequence();
 }
@@ -634,21 +644,21 @@ do_remote_cmd_only(const char *cmd, int nargs, ...)
   va_start(args, nargs);
 
   memcpy(printer_cmd + bufpos, cmd, 2);
-  STP_DEBUG(fprintf(stderr, "Remote command: %s", cmd));
+  STP_DEBUG(printf("Remote command: %s", cmd));
   bufpos += 2;
   printer_cmd[bufpos] = nargs % 256;
   printer_cmd[bufpos + 1] = (nargs >> 8) % 256;
-  STP_DEBUG(fprintf(stderr, " %02x %02x",
+  STP_DEBUG(printf(" %02x %02x",
 		    (unsigned) printer_cmd[bufpos],
 		    (unsigned) printer_cmd[bufpos + 1]));
   if (nargs > 0)
     for (i = 0; i < nargs; i++)
       {
 	printer_cmd[bufpos + 2 + i] = va_arg(args, int);
-	STP_DEBUG(fprintf(stderr, " %02x",
+	STP_DEBUG(printf(" %02x",
 			  (unsigned) printer_cmd[bufpos + 2 + i]));
       }
-  STP_DEBUG(fprintf(stderr, "\n"));
+  STP_DEBUG(printf("\n"));
   bufpos += 2 + nargs;
 }
 
@@ -656,7 +666,7 @@ static void
 add_resets(int count)
 {
   int i;
-  STP_DEBUG(fprintf(stderr, "Add %d resets\n", count));
+  STP_DEBUG(printf("Add %d resets\n", count));
   for (i = 0; i < count; i++)
     {
       printer_cmd[bufpos++] = '\033';
@@ -664,49 +674,77 @@ add_resets(int count)
     }
 }
 
+static int
+send_nulls(int fd)
+{
+  char buf[16384];
+  (void) memset(buf, 0, sizeof(buf));
+  (void) write(fd, buf, sizeof(buf));
+}
+
 static int
 init_packet(int fd, int force)
 {
   int status;
+  int tries = 0;
+  STP_DEBUG(printf("Init packet mode %d\n", force));
 
+ Loop:
   if (!force)
     {
-      STP_DEBUG(fprintf(stderr, "Flushing data...\n"));
+      STP_DEBUG(printf("Flushing early data...\n"));
       flushData(fd, (unsigned char) -1);
     }
 
-  STP_DEBUG(fprintf(stderr, "EnterIEEE...\n"));
+  STP_DEBUG(printf("EnterIEEE...\n"));
   if (!EnterIEEE(fd))
     {
+      STP_DEBUG(printf("EnterIEEE failed!\n"));
+      if (tries++ < 5)
+	{
+	  STP_DEBUG(printf("Retrying\n"));
+	  send_nulls(fd);
+	  flushData(fd, (unsigned char) -1);
+	  goto Loop;
+	}
       return 1;
     }
-  STP_DEBUG(fprintf(stderr, "Init...\n"));
+  STP_DEBUG(printf("Init...\n"));
   if (!Init(fd))
     {
+      STP_DEBUG(printf("Init failed!\n"));
+      if (tries++ < 5)
+	{
+	  STP_DEBUG(printf("Retrying\n"));
+	  send_nulls(fd);
+	  flushData(fd, (unsigned char) -1);
+	  goto Loop;
+	}
       return 1;
     }
 
-  STP_DEBUG(fprintf(stderr, "GetSocket...\n"));
+  STP_DEBUG(printf("GetSocket...\n"));
   socket_id = GetSocketID(fd, "EPSON-CTRL");
   if (!socket_id)
     {
+      STP_DEBUG(printf("GetSocket failed!\n"));
       return 1;
     }
-  STP_DEBUG(fprintf(stderr, "OpenChannel...\n"));
+  STP_DEBUG(printf("OpenChannel...\n"));
   switch ( OpenChannel(fd, socket_id, &send_size, &receive_size) )
     {
     case -1:
-      STP_DEBUG(fprintf(stderr,"Fatal Error return 1\n"));
+      STP_DEBUG(printf("Fatal Error return 1\n"));
       return 1; /* unrecoverable error */
       break;
     case  0:
-      STP_DEBUG(fprintf(stderr, "Error\n")); /* recoverable error ? */
+      STP_DEBUG(printf("Error\n")); /* recoverable error ? */
       return 1;
       break;
     }
 
   status = 1;
-  STP_DEBUG(fprintf(stderr, "Flushing data...\n"));
+  STP_DEBUG(printf("Flushing data...\n"));
   flushData(fd, socket_id);
   return 0;
 }
@@ -775,7 +813,7 @@ set_printer_model(void)
 	      if (desc.p_type == STP_PARAMETER_TYPE_BOOLEAN)
 		interchangeable_inks = desc.deflt.boolean;
 	      stp_parameter_description_destroy(&desc);
-	      STP_DEBUG(fprintf(stderr, "Found it! %s\n", printer_model));
+	      STP_DEBUG(printf("Found it! %s\n", printer_model));
 	      return;
 	    }
 	}
@@ -784,13 +822,32 @@ set_printer_model(void)
   the_printer_t = NULL;
 }
 
+static int
+test_for_di(const unsigned char *buf)
+{
+  return !(strncmp("di", (const char *)buf, 2) &&
+	   strncmp("@EJL ID", (const char *)buf, 7));
+}
+
+static int
+test_for_st(const unsigned char *buf)
+{
+  return !(strncmp("st", (const char *)buf, 2) &&
+	   strncmp("@BDC ST", (const char *)buf, 7));
+}
+
+static int
+test_for_ii(const unsigned char *buf)
+{
+  return !(strncmp("ii", (const char *)buf, 2) &&
+	   strncmp("@BDC PS", (const char *)buf, 7));
+}
+
 static const stp_printer_t *
 initialize_printer(int quiet, int fail_if_not_found)
 {
   int packet_initialized = 0;
   int fd;
-  int credit;
-  int retry = 4;
   int tries = 0;
   int status;
   int forced_packet_mode = 0;
@@ -813,14 +870,14 @@ initialize_printer(int quiet, int fail_if_not_found)
 	  status = SafeWrite(fd, init_str, sizeof(init_str) - 1);
 	  alarm(0);
 	  signal(SIGALRM, SIG_DFL);
-	  STP_DEBUG(fprintf(stderr, "status %d alarm %d\n", status, alarm_interrupt));
+	  STP_DEBUG(printf("status %d alarm %d\n", status, alarm_interrupt));
 	  if (status != sizeof(init_str) - 1 && (status != -1 || !alarm_interrupt))
 	    {
 	      fprintf(stderr, _("Cannot write to %s: %s\n"), raw_device,
 		      strerror(errno));
 	      exit(1);
 	    }
-	  STP_DEBUG(fprintf(stderr, "Try old command %d alarm %d\n",
+	  STP_DEBUG(printf("Try old command %d alarm %d\n",
 			    tries, alarm_interrupt));
 	  status = read_from_printer(fd, (char*)buf, 1024, 1);
 	  if (status <= 0 && tries > 0)
@@ -831,7 +888,7 @@ initialize_printer(int quiet, int fail_if_not_found)
 	  if (!forced_packet_mode &&
 	      status > 0 && !strstr((char *) buf, "@EJL ID") && tries < 3)
 	    {
-	      STP_DEBUG(fprintf(stderr, "Found bad data: %s\n", buf));
+	      STP_DEBUG(printf("Found bad data: %s\n", buf));
 	      /*
 	       * We know the printer's not dead.  Try to turn off status
 	       * and try again.
@@ -847,56 +904,47 @@ initialize_printer(int quiet, int fail_if_not_found)
 
       if (forced_packet_mode || ((buf[3] == status) && (buf[6] == 0x7f)))
 	{
-	  STP_DEBUG(fprintf(stderr, "Printer in packet mode....\n"));
+	  /*
+	   * NX100 looks like it's in D4 mode, but it doesn't really
+	   * respond correctly.  So we force it out of D4 mode and
+	   * then back in to ensure that it's right.  Trying to force
+	   * it into D4 mode alone isn't good enough.
+	   */
+	  initialize_print_cmd_new(0);
+	  (void) SafeWrite(fd, printer_cmd, bufpos);
+	  flushData(fd, (unsigned char) -1);
+	  forced_packet_mode = !init_packet(fd, 1);
+	  STP_DEBUG(printf("Printer in packet mode....\n"));
 	  packet_initialized = 1;
 	  isnew = 1;
-
-	  credit = askForCredit(fd, socket_id, &send_size, &receive_size);
-	  if (credit < 0)
-	    {
-	      STP_DEBUG(fprintf(stderr, "Cannot get credit\n"));
-	      return NULL;
-	    }
 	  /* request status command */
-	  status = writeData(fd, socket_id, (const unsigned char*)"di\1\0\1",
-			     5, 1);
+	  status =
+	    writeAndReadData(fd, socket_id, (const unsigned char*)"di\1\0\1",
+			     5, 1, (unsigned char *) buf, 1023,
+			     &send_size, &receive_size, &test_for_di);
 	  if (status <= 0)
 	    {
 	      fprintf(stderr, _("\nCannot write to %s: %s\n"),
 		      raw_device, strerror(errno));
-	      return NULL;
-	    }
-	  do
-	    {
-	      status = readData(fd, socket_id, (unsigned char*)buf, 1023);
-	      STP_DEBUG(fprintf(stderr, "readData try %d status %d\n",
-				retry, status));
-	      if (status <= -1 )
-		return NULL;
-	    }
-	  while ( (retry-- != 0) && strncmp("di", (char*)buf, 2) &&
-		  strncmp("@EJL ID", (char*)buf, 7));
-	  if (!retry)
-	    {
-	      STP_DEBUG(fprintf(stderr, "No retries left!\n"));
+	      close(fd);
 	      return NULL;
 	    }
 	}
-      STP_DEBUG(fprintf(stderr, "status: %i\n", status));
-      STP_DEBUG(fprintf(stderr, "Buf: %s\n", buf));
+      STP_DEBUG(printf("status: %i\n", status));
+      STP_DEBUG(printf("Buf: %s\n", buf));
       if (status > 0)
 	{
 	  pos = strstr((char*)buf, "@EJL ID");
-	  STP_DEBUG(fprintf(stderr, "pos: %s\n", pos));
+	  STP_DEBUG(printf("pos: %s\n", pos ? pos : "(null)"));
 	  if (pos)
 	    pos = strchr(pos, (int) ';');
-	  STP_DEBUG(fprintf(stderr, "pos: %s\n", pos));
+	  STP_DEBUG(printf("pos: %s\n", pos ? pos : "(null)"));
 	  if (pos)
 	    pos = strchr(pos + 1, (int) ';');
-	  STP_DEBUG(fprintf(stderr, "pos: %s\n", pos));
+	  STP_DEBUG(printf("pos: %s\n", pos ? pos : "(null)"));
 	  if (pos)
 	    pos = strchr(pos, (int) ':');
-	  STP_DEBUG(fprintf(stderr, "pos: %s\n", pos));
+	  STP_DEBUG(printf("pos: %s\n", pos ? pos : "(null)"));
 	  if (pos)
 	    {
 	      spos = strchr(pos, (int) ';');
@@ -924,12 +972,13 @@ initialize_printer(int quiet, int fail_if_not_found)
 		   * However, make it clear that this is a dummy,
 		   * so we don't actually try to print it out.
 		   */
-		  STP_DEBUG(fprintf(stderr, "Can't find printer name, assuming Stylus Photo\n"));
+		  STP_DEBUG(printf("Can't find printer name, assuming Stylus Photo\n"));
 		  printer_model = c_strdup("escp2-photo");
 		}
 	      else
 		{
-		  STP_DEBUG(fprintf(stderr, "Can't get response to @EJL ID\n"));
+		  STP_DEBUG(printf("Can't get response to @EJL ID\n"));
+		  close(fd);
 		  return NULL;
 		}
 	    }
@@ -938,7 +987,7 @@ initialize_printer(int quiet, int fail_if_not_found)
 	      if (spos)
 		*spos = '\000';
 	      printer_model = pos + 1;
-	      STP_DEBUG(fprintf(stderr, "printer model: %s\n", printer_model));
+	      STP_DEBUG(printf("printer model: %s\n", printer_model));
 	    }
 	}
     }
@@ -951,7 +1000,7 @@ initialize_printer(int quiet, int fail_if_not_found)
     }
 
   close(fd);
-  STP_DEBUG(fprintf(stderr, "new? %s\n", isnew ? "yes" : "no"));
+  STP_DEBUG(printf("new? %s\n", isnew ? "yes" : "no"));
   return the_printer_t;
 }
 
@@ -964,7 +1013,7 @@ get_printer(int quiet, int fail_if_not_found)
     {
       const stp_printer_t *printer =
 	initialize_printer(quiet, fail_if_not_found);
-      STP_DEBUG(fprintf(stderr, "init done, printer found? %s...\n",
+      STP_DEBUG(printf("init done, printer found? %s...\n",
 			printer ? "yes" : "no"));
       return printer;
     }
@@ -998,8 +1047,8 @@ static const char *aux_colors[] =
     N_("Cyan"),			/* 1 */
     N_("Magenta"),		/* 2 */
     N_("Yellow"),		/* 3 */
-    NULL,			/* 4 */
-    NULL,			/* 5 */
+    N_("Light Cyan"),		/* 4 */
+    N_("Light Magenta"),	/* 5 */
     NULL,			/* 6 */
     NULL,			/* 7 */
     NULL,			/* 8 */
@@ -1223,7 +1272,7 @@ print_old_ink_levels(const char *ind, stp_string_list_t *color_list)
       if (!ind[0] || ind[0] == ';')
 	return;
       val = (get_digit(ind[0]) << 4) + get_digit(ind[1]);
-      printf("%18s    %20d\n",
+      printf("%20s    %20d\n",
 	     gettext(stp_string_list_param(color_list, i)->text), val);
       ind += 2;
     }
@@ -1232,6 +1281,9 @@ print_old_ink_levels(const char *ind, stp_string_list_t *color_list)
 static void
 do_old_status(status_cmd_t cmd, const char *buf, const stp_printer_t *printer)
 {
+  if (cmd == CMD_STATUS)
+    printf(_("Printer Name: %s\n"),
+	   printer ? stp_printer_get_long_name(printer) : _("Unknown"));
   do
     {
       const char *ind;
@@ -1251,10 +1303,9 @@ do_old_status(status_cmd_t cmd, const char *buf, const stp_printer_t *printer)
 	      if (desc.p_type == STP_PARAMETER_TYPE_STRING_LIST)
 		{
 		  color_list = stp_string_list_create_copy(desc.bounds.str);
-		  STP_DEBUG(fprintf(stderr,
-				    "Using color list from driver (%ld %ld)\n",
-				    (long)stp_string_list_count(desc.bounds.str),
-				    (long)stp_string_list_count(color_list)));
+		  STP_DEBUG(printf("Using color list from driver (%ld %ld)\n",
+				   (long)stp_string_list_count(desc.bounds.str),
+				   (long)stp_string_list_count(color_list)));
 		  stp_parameter_description_destroy(&desc);
 		}
 	    }
@@ -1270,13 +1321,13 @@ do_old_status(status_cmd_t cmd, const char *buf, const stp_printer_t *printer)
 	  
 	  if (cmd == CMD_STATUS)
 	    printf(_("Ink Levels:\n"));
-	  printf("%18s    %20s\n", _("Ink color"), _("Percent remaining"));
+	  printf("%20s    %20s\n", _("Ink color"), _("Percent remaining"));
 	  print_old_ink_levels(ind, color_list);
 	  stp_string_list_destroy(color_list);
 	  if (cmd == CMD_STATUS)
 	    printf("\n");
 	}
-      STP_DEBUG(fprintf(stderr, "looking at %s\n", buf));
+      STP_DEBUG(printf("looking at %s\n", buf));
     } while ((buf = find_group(buf)) != NULL);
 }  
 
@@ -1293,7 +1344,7 @@ do_new_status(status_cmd_t cmd, char *buf, int bytes,
   stp_describe_parameter(printvars, "ChannelNames", &desc);
   if (desc.p_type == STP_PARAMETER_TYPE_STRING_LIST)
     color_list = desc.bounds.str;
-  STP_DEBUG(fprintf(stderr, "New format bytes: %d bytes\n", bytes));
+  STP_DEBUG(printf("New format bytes: %d bytes\n", bytes));
   if (cmd == CMD_STATUS)
     printf(_("Printer Name: %s\n"),
 	    printer ? stp_printer_get_long_name(printer) : _("Unknown"));
@@ -1302,28 +1353,40 @@ do_new_status(status_cmd_t cmd, char *buf, int bytes,
       unsigned hdr = buf[i];
       unsigned total_param_count = buf[i + 1];
       unsigned param = buf[i + 2];
-      STP_DEBUG(fprintf(stderr, "Header: %x param count: %d\n", hdr, total_param_count));
+      STP_DEBUG(printf("Header: %x param count: %d\n", hdr, total_param_count));
       if (hdr == 0x0f)	/* Always report ink */
 	{
 	  size_t count = (total_param_count - 1) / param;
 	  ind = buf + i + 3;
 	  if (cmd == CMD_STATUS)
 	    printf(_("Ink Levels:\n"));
-	  printf("%18s    %20s\n", _("Ink color"), _("Percent remaining"));
+	  printf("%20s    %20s\n", _("Ink color"), _("Percent remaining"));
 	  for (j = 0; j < count; j++)
 	    {
-	      if (ind[0] < color_count && param == 3 &&
+	      STP_DEBUG(printf("    Ink %d: ind[0] %d ind[1] %d ind[2] %d interchangeable %d param %d count %d aux %d\n",
+				j, ind[0], ind[1], ind[2], interchangeable_inks,
+				param, color_count, aux_color_count));
+	      if (ind[0] < color_count && param == 3 /* &&
 		  (interchangeable_inks || ind[1] >= aux_color_count ||
-		   ! aux_colors[(int) ind[1]]))
-		printf("%18s    %20d\n",
-		       gettext(colors_new[(int) ind[0]]), ind[2]);
+		   ! aux_colors[(int) ind[1]]) */)
+		{
+		  STP_DEBUG(printf("Case 0\n"));
+		  printf("%20s    %20d\n",
+			 gettext(colors_new[(int) ind[0]]), ind[2]);
+		}
 	      else if (ind[1] < aux_color_count && aux_colors[(int) ind[1]])
-		printf("%18s    %20d\n",
-		       gettext(aux_colors[(int) ind[1]]), ind[2]);
+		{
+		  STP_DEBUG(printf("Case 1\n"));
+		  printf("%20s    %20d\n",
+			 gettext(aux_colors[(int) ind[1]]), ind[2]);
+		}
 	      else
-		printf("%8s 0x%02x 0x%02x    %20d\n",
-		       _("Unknown"), (unsigned char) ind[0],
-		       (unsigned char) ind[1], ind[2]);
+		{
+		  STP_DEBUG(printf("Case 2\n"));
+		  printf("%8s 0x%02x 0x%02x    %20d\n",
+			 _("Unknown"), (unsigned char) ind[0],
+			 (unsigned char) ind[1], ind[2]);
+		}
 	      ind += param;
 	    }
 	  if (cmd == CMD_STATUS)
@@ -1374,7 +1437,6 @@ do_status_command_internal(status_cmd_t cmd)
 {
   int fd;
   int status;
-  int credit;
   int retry = 4;
   char buf[1024];
   const stp_printer_t *printer;
@@ -1395,10 +1457,10 @@ do_status_command_internal(status_cmd_t cmd)
       exit(1);
     }
 
-  STP_DEBUG(fprintf(stderr, "%s...\n", cmd_name));
+  STP_DEBUG(printf("%s...\n", cmd_name));
   printer = get_printer(1, 0);
   if (!found_unknown_old_printer)
-    STP_DEBUG(fprintf(stderr, "%s found %s%s\n", gettext(cmd_name),
+    STP_DEBUG(printf("%s found %s%s\n", gettext(cmd_name),
 		      printer ? stp_printer_get_long_name(printer) :
 		      printer_model ? printer_model : "(null)",
 		      printer ? "" : "(Unknown model)"));
@@ -1407,33 +1469,15 @@ do_status_command_internal(status_cmd_t cmd)
 
   if (isnew)
     {
-      credit = askForCredit(fd, socket_id, &send_size, &receive_size);
-      if (credit < 0)
-        {
-          STP_DEBUG(fprintf(stderr, "\nCannot get credit\n"));
-          exit(1);
-        }
       /* request status command */
-      status = writeData(fd, socket_id, (const unsigned char*)"st\1\0\1",
-			 5, 1);
+      status =
+	writeAndReadData(fd, socket_id, (const unsigned char*)"st\1\0\1",
+			 5, 1, (unsigned char *) buf, 1023,
+			 &send_size, &receive_size, &test_for_st);
       if (status <= 0)
 	{
 	  fprintf(stderr, _("\nCannot write to %s: %s\n"), raw_device, strerror(errno));
-	  exit(1);
-	}
-      do
-	{
-	  status = readData(fd, socket_id, (unsigned char*)buf, 1023);
-	  if (status < 0)
-	    {
-	      exit(1);
-	    }
-	  STP_DEBUG(fprintf(stderr, "readData try %d status %d\n", retry, status));
-	} while ((retry-- != 0) && strncmp("st", buf, 2) &&
-		 strncmp("@BDC ST", buf, 7));
-      /* "@BCD ST ST"  found */
-      if (!retry)
-	{
+	  CloseChannel(fd, socket_id);
 	  exit(1);
 	}
       buf[status] = '\0';
@@ -1482,8 +1526,6 @@ do_extended_ink_info(int extended_output)
 {
   int fd;
   int status;
-  int credit;
-  int retry = 4;
   char buf[1024];
   unsigned val, id, id2, year, year2, month, month2;
   unsigned iv[6];
@@ -1517,12 +1559,12 @@ do_extended_ink_info(int extended_output)
     {
       stp_string_list_t *color_list = stp_string_list_create();
 
-      if (printer)
+      if (printer && desc.p_type == STP_PARAMETER_TYPE_STRING_LIST)
 	{
-	  color_list = stp_string_list_create_copy(desc.bounds.str);
-	  STP_DEBUG(fprintf(stderr, "Using color list from driver (%ld %ld)\n",
+	  STP_DEBUG(printf("Using color list from driver (%ld %ld)\n",
 			    (long)stp_string_list_count(desc.bounds.str),
 			    (long)stp_string_list_count(color_list)));
+	  color_list = stp_string_list_create_copy(desc.bounds.str);
 	  stp_parameter_description_destroy(&desc);
 	}
       else
@@ -1533,16 +1575,11 @@ do_extended_ink_info(int extended_output)
 	   * message rather than from the ink list.  This gives us a
 	   * last chance to determine the inks
 	   */
-	  credit = askForCredit(fd, socket_id, &send_size, &receive_size);
-	  if (credit < 0)
-	    {
-	      stp_parameter_description_destroy(&desc);
-	      STP_DEBUG(fprintf(stderr, "Cannot get credit\n"));
-	      exit(1);
-	    }
 	  /* request status command */
-	  status = writeData(fd, socket_id, (const unsigned char*)"st\1\0\1",
-			     5, 1);
+	  status =
+	    writeAndReadData(fd, socket_id, (const unsigned char*)"st\1\0\1",
+			     5, 1, (unsigned char *) buf, 1023,
+			     &send_size, &receive_size, &test_for_st);
 	  if (status <= 0)
 	    {
 	      stp_parameter_description_destroy(&desc);
@@ -1550,29 +1587,10 @@ do_extended_ink_info(int extended_output)
 		      raw_device, strerror(errno));
 	      exit(1);
 	    }
-	  do
-	    {
-	      status = readData(fd, socket_id, (unsigned char*)buf, 1023);
-	      if (status < 0)
-		{
-		  stp_parameter_description_destroy(&desc);
-		  exit(1);
-		}
-	      STP_DEBUG(fprintf(stderr, "readData try %d status %d\n",
-				retry, status));
-	    }
-	  while ((retry-- != 0) && strncmp("st", buf, 2) &&
-		 strncmp("@BDC ST", buf, 7));
-	  /* "@BCD ST ST"  found */
-	  if (!retry)
-	    {
-	      stp_parameter_description_destroy(&desc);
-	      exit(1);
-	    }
 	  buf[status] = '\0';
 	  if ( buf[7] == '2' )
 	    {
-	      STP_DEBUG(fprintf(stderr, "New format ink!\n"));
+	      STP_DEBUG(printf("New format ink!\n"));
 	      /* new binary format ! */
 	      i = 10;
 	      while (buf[i] != 0x0f && i < status)
@@ -1582,48 +1600,52 @@ do_extended_ink_info(int extended_output)
 	      while (i < ind[1])
 		{
 		  if (ind[i] < color_count)
-		    stp_string_list_add_string(color_list,
-					       colors_new[(int) ind[i]],
-					       colors_new[(int) ind[i]]);
+		    {
+		      STP_DEBUG(printf("   Case 0: Ink %d %d (%s)\n",
+				       i, ind[i], colors_new[(int) ind[i]]));
+		      stp_string_list_add_string(color_list,
+						 colors_new[(int) ind[i]],
+						 colors_new[(int) ind[i]]);
+		    }
 		  else if (ind[i] == 0x40 && ind[i + 1] < aux_color_count)
-		    stp_string_list_add_string(color_list,
-					       aux_colors[(int) ind[i + 1]],
-					       aux_colors[(int) ind[i + 1]]);
+		    {
+		      STP_DEBUG(printf("   Case 1: Ink %d %d (%s)\n",
+				       i, ind[i+1], aux_colors[(int) ind[i+1]]));
+		      stp_string_list_add_string(color_list,
+						 aux_colors[(int) ind[i + 1]],
+						 aux_colors[(int) ind[i + 1]]);
+		    }
 		  else
-		    stp_string_list_add_string(color_list, "Unknown",
-					       "Unknown");
+		    {
+		      STP_DEBUG(printf("   Case 2: Unknown\n"));
+		      stp_string_list_add_string(color_list, "Unknown",
+						 "Unknown");
+		    }
 		  i+=3;
 		}
 	    }
-	  STP_DEBUG(fprintf(stderr, "Using color list from status message\n"));
+	  STP_DEBUG(printf("Using color list from status message\n"));
 	}
 
       for (i = 0; i < stp_string_list_count(color_list); i++)
         {
 	  char req[] = "ii\2\0\1\1";
-          credit = askForCredit(fd, socket_id, &send_size, &receive_size);
-	  if (credit < 0)
-	    exit(1);
 	  req[5] = i + 1;
-	  /* request status command */
-	  status = writeData(fd, socket_id, (const unsigned char*)req, 6, 1);
+	  status =
+	    writeAndReadData(fd, socket_id, (const unsigned char*)req,
+			     6, 1, (unsigned char *) buf, 1023,
+			     &send_size, &receive_size, &test_for_ii);
 	  if (status <= 0)
-	    exit(1);
-	  retry = 4;
-	  do
 	    {
-	      status = readData(fd, socket_id, (unsigned char*) buf, 1023);
-	      if (status < 0)
-		{
-		  exit(1);
-		}
-	    } while ((retry-- != 0) && strncmp("ii", buf, 2) &&
-		     strncmp("@BDC PS", buf, 7));
-	  if (!retry) /* couldn't read answer */
-	    exit(1);
+	      CloseChannel(fd, socket_id);
+	      exit(1);
+	    }
 	  ind = strchr(buf, 'I');
 	  if (!ind)
-	    printf("Cannot identify cartridge in slot %d\n", i);
+	    {
+	      STP_DEBUG(printf("Case 0: failure %i (%s)\n", i, buf));
+	      printf("Cannot identify cartridge in slot %d\n", i);
+	    }
 	  else if (sscanf(ind,
 			  "II:01;IQT:%x;TSH:%*4s;PDY:%x;PDM:%x;IC1:%x;IC2:%*x;IK1:%*x;IK2:%*x;TOV:%*x;TVU:%*x;LOG:EPSON;IQT:%x,%x,%x,%x,%x;TSH:%*4s;PDY:%x;PDM:%x;IC1:%x;IC2:%*xIK1:%*x;IK2;%*x;TOV:%*x;TVU:%*x;LOG:EPSON;",
 			  &iv[0], &year, &month, &id,
@@ -1636,14 +1658,17 @@ do_extended_ink_info(int extended_output)
 			  &year2, &month2, &id2) == 12)
 	    {
 	      int j;
-	      printf("%18s    %20s   %12s   %7s\n",
+	      STP_DEBUG(printf("Case 1: i %i iv %ud %ud %ud %ud %ud %ud year %ud %ud mo %ud %ud id %ud %ud\n",
+				i, iv[0], iv[1], iv[2], iv[3], iv[4], iv[5],
+				year, year2, month, month2, id, id2));
+	      printf("%20s    %20s   %12s   %7s\n",
 		     _("Ink cartridge"), _("Percent remaining"), _("Part number"),
 		     _("Date"));
-	      printf("%18s    %20d    T0%03d            %2d%02d-%02d\n",
+	      printf("%20s    %20d    T0%03d            %2d%02d-%02d\n",
 		     gettext(stp_string_list_param(color_list, 0)->text),
 		     iv[0], id, (year > 80 ? 19 : 20), year, month);
 	      for (j = 1; j < 6; j++)
-		printf("%18s    %20d    T0%03d            %2d%02d-%02d\n",
+		printf("%20s    %20d    T0%03d            %2d%02d-%02d\n",
 		       gettext(stp_string_list_param(color_list, j)->text),
 		       iv[j], id2, (year2 > 80 ? 19 : 20), year2, month2);
 	      break;
@@ -1660,14 +1685,17 @@ do_extended_ink_info(int extended_output)
 			  &year2, &month2, &id2) == 10)
 	    {
 	      int j;
-	      printf("%18s    %20s   %12s   %7s\n",
+	      STP_DEBUG(printf("Case 2: i %i iv %ud %ud %ud %ud year %ud %ud mo %ud %ud id %ud %ud\n",
+				i, iv[0], iv[1], iv[2], iv[3],
+				year, year2, month, month2, id, id2));
+	      printf("%20s    %20s   %12s   %7s\n",
 		     _("Ink cartridge"), _("Percent remaining"), _("Part number"),
 		     _("Date"));
-	      printf("%18s    %20d    T0%03d            %2d%02d-%02d\n",
+	      printf("%20s    %20d    T0%03d            %2d%02d-%02d\n",
 		     gettext(stp_string_list_param(color_list, 0)->text),
 		     iv[0], id, (year > 80 ? 19 : 20), year, month);
 	      for (j = 1; j < 4; j++)
-		printf("%18s    %20d    T0%03d            %2d%02d-%02d\n",
+		printf("%20s    %20d    T0%03d            %2d%02d-%02d\n",
 		       gettext(stp_string_list_param(color_list, j)->text),
 		       iv[j], id2, (year2 > 80 ? 19 : 20), year2, month2);
 	      break;
@@ -1679,16 +1707,36 @@ do_extended_ink_info(int extended_output)
 			  "II:01;IQT:%x;TSH:%*4s;PDY:%x;PDM:%x;IC1:%x;IC2:%*x;IK1:%*x;IK2:%*x;TOV:%*x;TVU:%*x;LOG:INKbyEPSON;",
 			  &val, &year, &month, &id ) == 4)
 	    {
+	      STP_DEBUG(printf("Case 3: i %i val %ud year %ud mo %ud id %ud\n",
+				i, val, year, month, id));
 	      if (i == 0)
-		printf("%18s    %20s   %12s   %7s\n",
+		printf("%20s    %20s   %12s   %7s\n",
 		       _("Ink cartridge"), _("Percent remaining"), _("Part number"),
 		       _("Date"));
-	      printf("%18s    %20d    T0%03d            %2d%02d-%02d\n",
+	      printf("%20s    %20d    T0%03d            %2d%02d-%02d\n",
+		     gettext(stp_string_list_param(color_list, i)->text),
+		     val, id, (year > 80 ? 19 : 20), year, month);
+	    }
+	  else if (sscanf(ind,
+			  "IQT:%x;TSH:%*4s;PDY:%x;PDM:%x;IC1:%x;IC2:%*x;IK1:%*x;IK2:%*x;TOV:%*x;TVU:%*x;LOG:EPSON;",
+			  &val, &year, &month, &id ) == 4 ||
+		   sscanf(ind,
+			  "IQT:%x;TSH:%*4s;PDY:%x;PDM:%x;IC1:%x;IC2:%*x;IK1:%*x;IK2:%*x;TOV:%*x;TVU:%*x;LOG:INKbyEPSON;",
+			  &val, &year, &month, &id ) == 4)
+	    {
+	      STP_DEBUG(printf("Case 4: i %i val %ud year %ud mo %ud id %ud\n",
+				i, val, year, month, id));
+	      if (i == 0)
+		printf("%20s    %20s   %12s   %7s\n",
+		       _("Ink cartridge"), _("Percent remaining"), _("Part number"),
+		       _("Date"));
+	      printf("%20s    %20d    T0%03d            %2d%02d-%02d\n",
 		     gettext(stp_string_list_param(color_list, i)->text),
 		     val, id, (year > 80 ? 19 : 20), year, month);
 	    }
 	  else
 	    {
+	      STP_DEBUG(printf("Case 5: failure %i (%s)\n", i, ind));
 	      printf("Cannot identify cartridge in slot %d\n", i);
 	    }
 	}
@@ -2131,7 +2179,7 @@ do_get_input (const char *prompt)
 	fgets_status = fgets (input, BUFSIZ, stdin);
 	if (fgets_status == NULL)
 	{
-		fprintf (stderr, _("Error in input\n"));
+		fprintf (stdout, _("Error in input\n"));
 		return (NULL);
 	}
 	else if (strlen (input) == 1 && input[0] == '\n')
