@@ -1,5 +1,5 @@
 /*
- * "$Id: print-vars.c,v 1.89 2008/07/21 00:02:32 rlk Exp $"
+ * "$Id: print-vars.c,v 1.91 2010/12/05 21:38:15 rlk Exp $"
  *
  *   Print plug-in driver utility functions for the GIMP.
  *
@@ -84,16 +84,34 @@ struct stp_vars			/* Plug-in variables */
 
 static int standard_vars_initialized = 0;
 
+
+void
+stp_parameter_description_destroy(stp_parameter_t *desc)
+{
+  switch (desc->p_type)
+    {
+    case STP_PARAMETER_TYPE_CURVE:
+      if (desc->bounds.curve)
+	stp_curve_destroy(desc->bounds.curve);
+      desc->bounds.curve = NULL;
+      break;
+    case STP_PARAMETER_TYPE_ARRAY:
+      if (desc->bounds.array)
+	stp_array_destroy(desc->bounds.array);
+      desc->bounds.array = NULL;
+      break;
+    case STP_PARAMETER_TYPE_STRING_LIST:
+      if (desc->bounds.str)
+	stp_string_list_destroy(desc->bounds.str);
+      desc->bounds.str = NULL;
+      break;
+    default:
+      break;
+    }
+}
 static stp_vars_t default_vars;
 
-#define CHECK_VARS(v)							\
-do {									\
-  if (v == NULL)							\
-    {									\
-      stp_erprintf("Null stp_vars_t! Please report this bug.\n");	\
-      stp_abort();							\
-    }									\
-} while (0)
+#define CHECK_VARS(v) STPI_ASSERT(v, NULL)
 
 static const char *
 value_namefunc(const void *item)
@@ -1397,17 +1415,9 @@ stp_fill_parameter_settings(stp_parameter_t *desc,
       desc->verify_this_parameter = param->verify_this_parameter;
       desc->read_only = param->read_only;
       desc->name = param->name;
-      if (!(param->text))
-	{
-	  stp_erprintf("No text string for parameter %s!\n", desc->name);
-	  stp_abort();
-	}
+      STPI_ASSERT(param->text, NULL);
       desc->text = gettext(param->text);
-      if (!(param->category))
-	{
-	  stp_erprintf("No category string for parameter %s!\n", desc->name);
-	  stp_abort();
-	}
+      STPI_ASSERT(param->category, NULL);
       desc->category = gettext(param->category);
       desc->help = param->help ? gettext(param->help) : NULL;
       return;
@@ -1617,29 +1627,103 @@ stp_describe_parameter(const stp_vars_t *v, const char *name,
     stp_deprintf(STP_DBG_VARS, "Describing invalid parameter %s\n", name);
 }
 
-void
-stp_parameter_description_destroy(stp_parameter_t *desc)
+stp_string_list_t *
+stp_parameter_get_categories(const stp_vars_t *v, const stp_parameter_t *desc)
 {
-  switch (desc->p_type)
+  const char *dptr;
+  stp_string_list_t *answer;
+  int count = 0;
+  if (!v || !desc || !(desc->category))
+    return NULL;
+  answer = stp_string_list_create();
+  dptr = desc->category;
+  while (dptr)
     {
-    case STP_PARAMETER_TYPE_CURVE:
-      if (desc->bounds.curve)
-	stp_curve_destroy(desc->bounds.curve);
-      desc->bounds.curve = NULL;
-      break;
-    case STP_PARAMETER_TYPE_ARRAY:
-      if (desc->bounds.array)
-	stp_array_destroy(desc->bounds.array);
-      desc->bounds.array = NULL;
-      break;
-    case STP_PARAMETER_TYPE_STRING_LIST:
-      if (desc->bounds.str)
-	stp_string_list_destroy(desc->bounds.str);
-      desc->bounds.str = NULL;
-      break;
-    default:
-      break;
+      const char *xptr = strchr(dptr, '=');
+      if (xptr)
+	{
+	  char *name = stp_strndup(dptr, xptr - dptr);
+	  char *text;
+	  dptr = xptr + 1;
+	  xptr = strchr(dptr, ',');
+	  if (xptr)
+	    {
+	      text = stp_strndup(dptr, xptr - dptr);
+	      dptr = xptr + 1;
+	    }
+	  else
+	    {
+	      text = stp_strdup(dptr);
+	      dptr = NULL;
+	    }
+	  stp_string_list_add_string(answer, name, text);
+	  stp_free(name);
+	  stp_free(text);
+	  count++;
+	}
+      else
+	dptr = NULL;
     }
+  if (count == 0)
+    {
+      stp_string_list_destroy(answer);
+      return NULL;
+    }
+  else
+    return answer;
+}
+
+char *
+stp_parameter_get_category(const stp_vars_t *v, const stp_parameter_t *desc,
+			   const char *category)
+{
+  const char *dptr;
+  char *cptr;
+  int len;
+  if (!v || !desc || !(desc->category) || !category)
+    return NULL;
+  dptr = desc->category;
+  stp_asprintf(&cptr, "%s=", category);
+  len = stp_strlen(cptr);
+  while (dptr)
+    {
+      if (strncmp(dptr, cptr, len) == 0)
+	{
+	  const char *xptr;
+	  char *answer;
+	  dptr += len;
+	  xptr = strchr(dptr, ',');
+	  if (xptr)
+	    answer = stp_strndup(dptr, xptr - dptr);
+	  else
+	    answer = stp_strdup(dptr);
+	  stp_free(cptr);
+	  return answer;
+	}
+      dptr = strchr(dptr, ',');
+      if (dptr)
+	dptr++;
+    }
+  return NULL;
+}
+
+int
+stp_parameter_has_category_value(const stp_vars_t *v,
+				 const stp_parameter_t *desc,
+				 const char *category, const char *value)
+{
+  const char *dptr;
+  char *cptr;
+  int answer = 0;
+  if (!v || !desc || !category)
+    return -1;
+  cptr = stp_parameter_get_category(v, desc, category);
+  if (cptr == NULL)
+    return 0;
+  if (value == NULL || strcmp(value, cptr) == 0)
+    answer = 1;
+  stp_free(cptr);
+  return answer;
 }
 
 const stp_parameter_t *
@@ -1743,15 +1827,9 @@ fill_vars_from_xmltree(stp_mxml_node_t *prop, stp_mxml_node_t *root,
 		  cnode = stp_mxmlFindElement(root, root, "namedParam",
 					      "name", cref,
 					      STP_MXML_DESCEND);
-		  if (!cnode || cnode->type != STP_MXML_ELEMENT ||
-		      !(cnode->child))
-		    {
-		      stp_erprintf("Found parameter name %s type %s ref %s but no target!\n",
-				   p_name, p_type, cref);
-		      stp_abort();
-		    }
-		  else
-		    stp_deprintf(STP_DBG_XML, "Found parameter ref %s\n", cref);
+		  STPI_ASSERT(cnode && cnode->type == STP_MXML_ELEMENT &&
+			 cnode->child, v);
+		  stp_deprintf(STP_DBG_XML, "Found parameter ref %s\n", cref);
 		  cnode = cnode->child;
 		}
 	      if (strcmp(p_type, "float") == 0)
@@ -1853,54 +1931,32 @@ fill_vars_from_xmltree(stp_mxml_node_t *prop, stp_mxml_node_t *root,
 		  stp_curve_t *curve;
 		  while (cnode->type != STP_MXML_ELEMENT && cnode->next)
 		    cnode = cnode->next;
-		  if (!cnode)
-		    {
-		      stp_erprintf("Unable to find XML curve!\n");
-		      stp_abort();
-		    }
+		  STPI_ASSERT(cnode, v);
 		  curve = stp_curve_create_from_xmltree(cnode);
-		  if (curve)
+		  STPI_ASSERT(curve, v);
+		  stp_set_curve_parameter(v, p_name, curve);
+		  type = STP_PARAMETER_TYPE_DOUBLE;
+		  if (stp_get_debug_level() & STP_DBG_XML)
 		    {
-		      stp_set_curve_parameter(v, p_name, curve);
-		      type = STP_PARAMETER_TYPE_DOUBLE;
-		      if (stp_get_debug_level() & STP_DBG_XML)
-			{
-			  char *cv = stp_curve_write_string(curve);
-			  stp_deprintf(STP_DBG_XML, "  Set curve '%s' (%s)\n",
-				       p_name, cv);
-			  stp_free(cv);
-			}
-		      stp_curve_destroy(curve);
-		    }
-		  else
-		    {
-		      stp_erprintf("Unable to extract curve!\n");
-		      stp_abort();
+		      char *cv = stp_curve_write_string(curve);
+		      stp_deprintf(STP_DBG_XML, "  Set curve '%s' (%s)\n",
+				   p_name, cv);
+		      stp_free(cv);
 		    }
+		  stp_curve_destroy(curve);
 		}
 	      else if (strcmp(p_type, "array") == 0)
 		{
 		  stp_array_t *array;
 		  while (cnode->type != STP_MXML_ELEMENT && cnode->next)
 		    cnode = cnode->next;
-		  if (!cnode)
-		    {
-		      stp_erprintf("Unable to find XML array!\n");
-		      stp_abort();
-		    }
+		  STPI_ASSERT(cnode, v);
 		  array = stp_array_create_from_xmltree(cnode);
-		  if (array)
-		    {
-		      type = STP_PARAMETER_TYPE_DOUBLE;
-		      stp_set_array_parameter(v, p_name, array);
-		      stp_deprintf(STP_DBG_XML, "  Set array '%s'\n", p_name);
-		      stp_array_destroy(array);
-		    }
-		  else
-		    {
-		      stp_erprintf("Unable to extract array!\n");
-		      stp_abort();
-		    }
+		  STPI_ASSERT(array, v);
+		  type = STP_PARAMETER_TYPE_DOUBLE;
+		  stp_set_array_parameter(v, p_name, array);
+		  stp_deprintf(STP_DBG_XML, "  Set array '%s'\n", p_name);
+		  stp_array_destroy(array);
 		}
 	      else
 		{
