@@ -1,5 +1,5 @@
 /*
- * "$Id: print-weave.c,v 1.72 2008/10/29 00:10:42 easysw Exp $"
+ * "$Id: print-weave.c,v 1.76 2010/08/04 00:33:57 rlk Exp $"
  *
  *   Softweave calculator for Gutenprint.
  *
@@ -36,25 +36,6 @@
 #include <limits.h>
 #endif
 
-#if 1
-#define ASSERTIONS
-#endif
-
-#ifdef ASSERTIONS
-#define assert(x,v)							\
-do									\
-{									\
-  if (!(x))								\
-    {									\
-      stp_eprintf(v, "Assertion %s failed! file %s, line %d.\n",	\
-		  #x, __FILE__, __LINE__);				\
-      stp_abort();							\
-    }									\
-} while (0)
-#else
-#define assert(x,v) do {} while (0)
-#endif
-
 static int
 gcd(int x, int y)
 {
@@ -453,8 +434,8 @@ make_passmap(raw_t *w, int **map, int **starts, int first_pass_number,
 	int passes_to_map = first_pass_after_map - first_pass_to_map;
 	int i;
 
-	assert(first_pass_to_map <= first_pass_after_map, w->v);
-	assert(first_pass_to_stagger <= first_pass_after_stagger, w->v);
+	STPI_ASSERT(first_pass_to_map <= first_pass_after_map, w->v);
+	STPI_ASSERT(first_pass_to_stagger <= first_pass_after_stagger, w->v);
 
 	*map = passmap = stp_malloc(passes_to_map * sizeof(int));
 	*starts = startrows = stp_malloc(passes_to_map * sizeof(int));
@@ -646,8 +627,8 @@ stpi_calculate_row_parameters(void *vw,		/* I - weave parameters */
 	int stagger = 0;
 	int extra;
 
-	assert(row >= w->first_row_printed, w->rw.v);
-	assert(row <= w->last_row_printed, w->rw.v);
+	STPI_ASSERT(row >= w->first_row_printed, w->rw.v);
+	STPI_ASSERT(row <= w->last_row_printed, w->rw.v);
 	calculate_raw_row_parameters(&w->rw,
 	                             row + w->rw.separation * w->rw.jets,
 	                             subpass, &raw_pass, &jet, &startrow);
@@ -656,11 +637,11 @@ stpi_calculate_row_parameters(void *vw,		/* I - weave parameters */
 	phantomrows = 0;
 
 	if (raw_pass < w->first_normal_pass) {
-	        assert(raw_pass >= w->first_premapped_pass, w->rw.v);
+	        STPI_ASSERT(raw_pass >= w->first_premapped_pass, w->rw.v);
 		*pass = w->pass_premap[raw_pass - w->first_premapped_pass];
 		stagger = w->stagger_premap[raw_pass - w->first_premapped_pass];
 	} else if (raw_pass >= w->first_postmapped_pass) {
-	        assert(raw_pass >= w->first_postmapped_pass, w->rw.v);
+	        STPI_ASSERT(raw_pass >= w->first_postmapped_pass, w->rw.v);
 		*pass = w->pass_postmap[raw_pass - w->first_postmapped_pass];
 		stagger = w->stagger_postmap[raw_pass
 		                             - w->first_postmapped_pass];
@@ -1149,6 +1130,10 @@ stp_initialize_weave(stp_vars_t *v,
   sw->horizontal_width =
     (sw->compute_linewidth)(v, ((sw->linewidth + sw->horizontal_weave - 1) /
 				 sw->horizontal_weave));
+  stp_dprintf(STP_DBG_WEAVE_PARAMS, v,
+	      "Computing linewidth for linewidth %d weave %d => %d (%d)\n",
+	      sw->linewidth, sw->horizontal_weave, sw->horizontal_width,
+	      ((sw->horizontal_width + 7) / 8));
   sw->horizontal_width = ((sw->horizontal_width + 7) / 8);
 
   for (i = 0; i < sw->vmod; i++)
@@ -1173,6 +1158,10 @@ stp_initialize_weave(stp_vars_t *v,
 	      sw->oversample, sw->vmod,
 	      sw->vmod * sw->virtual_jets * sw->bitwidth *
 	      sw->ncolors * sw->horizontal_width, page_height);
+  stp_dprintf(STP_DBG_WEAVE_PARAMS, v,
+	      "      ***Buffer limit %d (vj %d bw %d hw %d)\n",
+	      sw->virtual_jets * sw->bitwidth * sw->horizontal_width,
+	      sw->virtual_jets, sw->bitwidth, sw->horizontal_width);
   stp_allocate_component_data(v, "Weave", NULL, stpi_destroy_weave, sw);
   return;
 }
@@ -1499,9 +1488,44 @@ add_to_row(stp_vars_t *v, stpi_softweave_t *sw, int row, unsigned char *buf,
   size_t count = linecount->v[color];
   if (place + nbytes > sw->virtual_jets * sw->bitwidth * sw->horizontal_width)
     {
-      stp_eprintf(v, "Buffer overflow: limit %d, actual %ld, count %ld\n",
+      int i;
+      stp_eprintf(v, "ERROR: %s\n", _("Fatal error!"));
+      stp_eprintf(v, "ERROR: Static weave data follows:\n");
+      stp_eprintf(v, "ERROR:    jets: %d\n", sw->jets);
+      stp_eprintf(v, "ERROR:    virtual_jets: %d\n", sw->virtual_jets);
+      stp_eprintf(v, "ERROR:    separation: %d\n", sw->separation);
+      stp_eprintf(v, "ERROR:    horizontal_weave: %d\n", sw->horizontal_weave);
+      stp_eprintf(v, "ERROR:    vertical_subpasses: %d\n", sw->vertical_subpasses);
+      stp_eprintf(v, "ERROR:    vmod: %d\n", sw->vmod);
+      stp_eprintf(v, "ERROR:    oversample: %d\n", sw->oversample);
+      stp_eprintf(v, "ERROR:    repeat_count: %d\n", sw->repeat_count);
+      stp_eprintf(v, "ERROR:    ncolors: %d\n", sw->ncolors);
+      stp_eprintf(v, "ERROR:    linewidth: %d\n", sw->linewidth);
+      stp_eprintf(v, "ERROR:    vertical_height: %d\n", sw->vertical_height);
+      stp_eprintf(v, "ERROR:    firstline: %d\n", sw->firstline);
+      stp_eprintf(v, "ERROR:    bitwidth: %d\n", sw->bitwidth);
+      stp_eprintf(v, "ERROR:    vertical_oversample: %d\n", sw->vertical_oversample);
+      stp_eprintf(v, "ERROR:    horizontal_width: %d\n", sw->horizontal_width);
+      if (sw->head_offset)
+	{
+	  stp_eprintf(v, "ERROR:    head_offset:\n");
+	  for (i = 0; i < sw->ncolors; i++)
+	    stp_eprintf(v, "ERROR:      head %d: %d\n", i, sw->head_offset[i]);
+	}
+      stp_eprintf(v, "ERROR: Dynamic weave data follows:\n");
+      stp_eprintf(v, "ERROR:    last_pass_offset: %d\n", sw->last_pass_offset);
+      stp_eprintf(v, "ERROR:    last_pass: %d\n", sw->last_pass);
+      stp_eprintf(v, "ERROR:    lineno: %d\n", sw->lineno);
+      stp_eprintf(v, "ERROR:    current_vertical_subpass: %d\n", sw->current_vertical_subpass);
+      stp_eprintf(v, "ERROR:    rcache: %d\n", sw->rcache);
+      stp_eprintf(v, "ERROR:    vcache: %d\n", sw->vcache);
+      stp_eprintf(v, "ERROR: Other parameters: row %d color %d setactive %d hpass %d\n",
+		  row, color, setactive, h_pass);
+      stp_eprintf(v, "ERROR: Buffer overflow: limit %d (jets %d bits %d horizontal %d), actual %ld (current %d added %d), count %ld\n",
 		  sw->virtual_jets * sw->bitwidth * sw->horizontal_width,
-		  (long)(place + nbytes), (long)count);
+		  sw->virtual_jets, sw->bitwidth, sw->horizontal_width,
+		  (long) (place + nbytes), (int) place, (int) nbytes, (long) count);
+      stp_eprintf(v, "ERROR: %s\n", _("Please report the above information to gimp-print-devel@lists.sourceforge.net"));
       stp_abort();
     }
   memcpy(bufs->v[color] + lineoffs->v[color], buf, nbytes);
@@ -1582,10 +1606,23 @@ stp_write_weave(stp_vars_t *v, unsigned char *const cols[])
   int cpass = sw->current_vertical_subpass * h_passes;
 
   if (!sw->fold_buf)
-    sw->fold_buf = stp_zalloc(sw->bitwidth * ylength);
+    {
+      stp_dprintf(STP_DBG_WEAVE_PARAMS, v,
+		  "fold buffer allocation: length %d lw %d weave %d xlength %d ylength %d\n",
+		  length, sw->linewidth, sw->horizontal_weave, xlength, ylength);
+      stp_dprintf(STP_DBG_WEAVE_PARAMS, v,
+		  "Allocating fold buf %d * %d (%d)\n", ylength, sw->bitwidth,
+		  sw->bitwidth * ylength);
+      sw->fold_buf = stp_zalloc(sw->bitwidth * ylength);
+    }
   if (!sw->comp_buf)
-    sw->comp_buf = stp_zalloc(sw->bitwidth *
-			      (sw->compute_linewidth)(v,ylength));
+    {
+      stp_dprintf(STP_DBG_WEAVE_PARAMS, v,
+		  "Allocating compression buffer based on %d, %d\n",
+		  sw->bitwidth, ylength);
+      sw->comp_buf = stp_zalloc(sw->bitwidth *
+				(sw->compute_linewidth)(v,ylength));
+    }
   if (sw->current_vertical_subpass == 0)
     initialize_row(v, sw, sw->lineno, xlength, cols);
 
@@ -1622,47 +1659,15 @@ stp_write_weave(stp_vars_t *v, unsigned char *const cols[])
 	    }
 	  else
 	    in = cols[j];
-	  switch (sw->horizontal_weave)
+	  if (sw->horizontal_weave == 1)
+	    memcpy(sw->s[0], in, length * sw->bitwidth);
+	  else
+	    stp_unpack(length, sw->bitwidth, sw->horizontal_weave, in, sw->s);
+	  if (sw->vertical_subpasses > 1)
 	    {
-	    case 1:
-	      memcpy(sw->s[0], in, length * sw->bitwidth);
-	      break;
-	    case 2:
-	      stp_unpack_2(length, sw->bitwidth, in, sw->s[0], sw->s[1]);
-	      break;
-	    case 4:
-	      stp_unpack_4(length, sw->bitwidth, in,
-			   sw->s[0], sw->s[1], sw->s[2], sw->s[3]);
-	      break;
-	    case 8:
-	      stp_unpack_8(length, sw->bitwidth, in,
-			   sw->s[0], sw->s[1], sw->s[2], sw->s[3],
-			   sw->s[4], sw->s[5], sw->s[6], sw->s[7]);
-	      break;
-	    case 16:
-	      stp_unpack_16(length, sw->bitwidth, in,
-			    sw->s[0], sw->s[1], sw->s[2], sw->s[3],
-			    sw->s[4], sw->s[5], sw->s[6], sw->s[7],
-			    sw->s[8], sw->s[9], sw->s[10], sw->s[11],
-			    sw->s[12], sw->s[13], sw->s[14], sw->s[15]);
-	      break;
-	    }
-	  switch (sw->vertical_subpasses)
-	    {
-	    case 4:
 	      for (idx = 0; idx < sw->horizontal_weave; idx++)
-		stp_split_4(length, sw->bitwidth, sw->s[idx], sw->s[idx],
-			    sw->s[idx + sw->horizontal_weave],
-			    sw->s[idx + sw->horizontal_weave * 2],
-			    sw->s[idx + sw->horizontal_weave * 3]);
-	      break;
-	    case 2:
-	      for (idx = 0; idx < sw->horizontal_weave; idx++)
-		stp_split_2(length, sw->bitwidth, sw->s[idx], sw->s[idx],
-			    sw->s[idx + sw->horizontal_weave]);
-	      break;
-	      /* case 1 is taken care of because the various unpack */
-	      /* functions will do the trick themselves */
+		stp_split(length, sw->bitwidth, sw->vertical_subpasses,
+			  sw->s[idx], sw->horizontal_weave, &(sw->s[idx]));
 	    }
 	  for (i = 0; i < h_passes; i++)
 	    {
